"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const node_path_1 = require("node:path");
const build_kit_1 = require("@genesislcap/build-kit");
exports.default = (ctx) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const { dirs: { cwd }, cli: { isTest, options }, pkg, } = ctx;
    const { coverage, watch, debug, e2e, interactive, browser } = options;
    if (!isTest) {
        throw new Error(`Unrecognized command: ${JSON.stringify(options)}`);
    }
    if (coverage) {
        yield (0, build_kit_1.rm)(['coverage']);
    }
    // E2E tests
    if (e2e) {
        const uiParam = interactive ? '--ui' : '';
        const debugParam = debug ? '--debug' : '';
        (0, build_kit_1.run)(cwd, `npx playwright test --config=./playwright.config.ts ${uiParam} ${debugParam}`, {
            prefixCommandWithNode: false,
            env: Object.assign(Object.assign({}, process.env), { PORT: (_a = pkg.config) === null || _a === void 0 ? void 0 : _a.PORT }),
        });
        return;
    }
    else {
        // unit tests
        if (browser) {
            // browser target - executing using playwright-test
            const playwrightTest = yield (0, build_kit_1.resolveBin)('playwright-test');
            const debugParam = debug ? '--debug' : '';
            const watchParam = watch ? '--watch' : '';
            const covParam = coverage ? '--cov' : '';
            const command = `${playwrightTest} "./**/*.test.ts"  --runner uvu ${debugParam} ${watchParam} ${covParam}`;
            (0, build_kit_1.run)(cwd, command);
            if (coverage) {
                const nyc = yield (0, build_kit_1.resolveBin)('nyc');
                const nycCommand = `${nyc} report --reporter=text --reporter=lcov`;
                (0, build_kit_1.run)(cwd, nycCommand);
            }
        }
        else {
            // node target - executing using uvu/jsdom
            const nodeArgs = debug ? '--inspect-brk' : '';
            // tsm expects a relative path to config file incl. extension
            const tsmJS = (0, node_path_1.resolve)(__dirname, './config/tsm.js');
            const tsmJSRelative = (0, node_path_1.relative)(cwd, tsmJS);
            const jsdomSetupPath = (0, node_path_1.resolve)(__dirname, './config/jsdom.setup.ts');
            const uvu = yield (0, build_kit_1.resolveBin)('uvu');
            const ignore = ['dist', 'e2e', 'docs', 'temp', 'coverage']; // .git, node_modules are ignored by default
            const unitTestCommand = `${uvu} -r tsm --tsmconfig ${tsmJSRelative} -r @esbuild-kit/cjs-loader -r ${jsdomSetupPath} . "test.ts" ${ignore.map((i) => ' -i ' + i)}`;
            if (coverage) {
                const c8 = yield (0, build_kit_1.resolveBin)('c8');
                const command = `${c8} --include=src --all  --reporter=text --reporter=lcov node ${unitTestCommand}`;
                (0, build_kit_1.run)(cwd, command);
            }
            else if (watch) {
                const watchlist = yield (0, build_kit_1.resolveBin)('watchlist');
                (0, build_kit_1.run)(cwd, `${watchlist} src test -- ${unitTestCommand}`);
            }
            else {
                (0, build_kit_1.run)(cwd, unitTestCommand, { nodeArgs });
            }
        }
    }
});
